# -*- coding: utf-8 -*-

import sqlalchemy as sa
from efa.lib.esferror import render_esf_errors, exist_complete_and_valid_oi_in
from efa.lib.validators import build_data_to_validate
from efa.model.participation import (
    Participation, ParticipationFactory,
    get_esfindicator_fields, get_values,
    listener_adapter, generate_determine_reportable_function_for_participation
)
from euhu.programs.waskiq import validate
from ringo.lib.helpers import HTML, literal
from ringo.lib.sql import DBSession
from ringo.model.base import BaseFactory
from waskiq.model.participant import derive_indicators, recipe, program

MIKROPROJECT_ID = 1
CM_ID = 2
CLEARING_ID = 3
JSA_ID = 4


def status_oi_logic(participation):
    if not participation.start_date:
        return "no_startdate"
    before_participations = DBSession.query(Participation) \
        .filter(WaskiqParticipation.participant_id ==
                participation.participant_id) \
        .filter(WaskiqParticipation.start_date <
                participation.start_date).all()
    current_complete_and_valid = exist_complete_and_valid_oi_in(
        [participation])
    before_complete_and_valid = exist_complete_and_valid_oi_in(
        before_participations, no_recalc=True)
    if current_complete_and_valid:
        return "current_complete_and_valid"
    elif before_complete_and_valid and not participation.oi:
        return "before_complete_and_valid_and_not_oi"
    elif not before_complete_and_valid and not participation.oi:
        return "not_before_complete_and_valid_and_not_oi"
    elif not before_participations and participation.oi \
            and not current_complete_and_valid:
        return ("not_before_participations_and_oi_and"
                "_not_current_complete_and_valid")
    elif participation.oi and not current_complete_and_valid:
        return "oi_and_not_current_complete_and_valid"
    elif not before_participations and not current_complete_and_valid:
        return "not_before_participations_and_not_current_complete_and_valid"
    return "else_path"


class WaskiqParticipationFactory(ParticipationFactory):
    def create(self, user=None, values=None):
        participation = BaseFactory.create(self, user, values)
        participation = self._add_result_indicators(participation, user)
        return participation


class WaskiqParticipation(Participation):
    """Waskiq specific Participation. Adds some local fields.
    """

    waskiqproject = sa.orm.relation("WaskiqProject")
    """waskiq specific relation to the :class:`.WaskiqProject`.
    Needed in form to get access to waskiq specific attributes of the
    Project. An alternativ is to overwrite the still existing
    `project` relation. This will trigger a warning on application start.
    But it seem not to cause any problems."""

    data_collection_agreement = sa.Column('data_collection_agreement',
                                          sa.Integer, nullable=False)
    '''
    Waskiq has different versions of the data collection agreement. To allow
    for some kind of tracking of what version of the agreement was used
    for each participation, the field was moved here.
    '''

    @classmethod
    def get_item_factory(cls, request=None):
        return WaskiqParticipationFactory(WaskiqParticipation, request)

    @property
    def baustein(self):
        result = []
        if self.baustein_cms:
            result.append(str(CM_ID))
        if self.baustein_mps:
            result.append(str(MIKROPROJECT_ID))
        if self.baustein_clearings:
            result.append(str(CLEARING_ID))
        if self.baustein_jsas:
            result.append(str(JSA_ID))
        return result

    @baustein.setter
    def baustein(self, value):
        pass

    @property
    def baustein_list(self):
        """AJSA=>AJS due to Issue 801"""
        dict1 = {
            '1': 'MP',
            '2': 'CM',
            '3': 'CL',
            '4': 'AJS'
        }
        return map(dict1.get, self.baustein)

    @property
    def render_oi_status(self):
        if self.oi is not None and self.oi.participant is not None:
            oi = getattr(self, "waskiqoi")
            if oi.is_complete():
                return HTML.tag("span", class_="glyphicon glyphicon-ok")
            else:
                return HTML.tag("span",
                                class_="glyphicon glyphicon-exclamation-sign")
        return u""

    @property
    def esf_errors(self):
        # Finally result in this after the long way of issue1800.
        oktext = HTML.tag("abbr", class_="text-success", title="auswertbar",
                          style="border-bottom:none;text-decoration:none",
                          c=u'✔')
        fehlertext = HTML.tag("abbr",
                              title="fehlerhaft",
                              class_="text-danger",
                              style=("font-weight: bold;border-bottom:none;"
                                     "text-decoration:none"),
                              c=u'✖')
        nichtauswertbartext = HTML.tag("abbr",
                                       title="nicht auswertbar",
                                       class_="text-warning",
                                       style=("font-weight: bold;"
                                              "border-bottom:none;"
                                              "text-decoration:none"),
                                       c=u'❇')
        out = render_esf_errors(self, WaskiqParticipation,
                                OK=oktext,
                                NICHTAUSWERTBAR=nichtauswertbartext,
                                FEHLER=fehlertext,
                                )
        return literal("").join(out)

    def validate(self):
        """Hook for using the validate function in efa."""
        esf_fields = get_esfindicator_fields(self)
        if not self.oi:
            esf_fields = filter(lambda f: not f.startswith("waskiqoi."),
                                esf_fields)
        esf_values = get_values(self, esf_fields)
        return validate(build_data_to_validate(esf_values, self))

    @property
    def form_values(self):
        values = self.get_values()
        if self.waskiqoi:
            values.update(self.waskiqoi.get_values())
        if self.waskiqri:
            values.update(self.waskiqri.get_values())
        values['birthday'] = self.participant.birthday
        values['gender'] = self.participant.gender
        return values


tn_update_listener = listener_adapter(
    generate_determine_reportable_function_for_participation(
        derive_indicators,
        recipe,
        validate,
        program))

sa.event.listen(WaskiqParticipation, 'before_update', tn_update_listener)
